#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.

"""
Resilient utility to configure secrets in the config file (~/.resilient/app.config)
"""

from __future__ import print_function
import sys
import os
import logging
import keyring
import resilient
from six import string_types

try:
    # For all python < 3.2
    from io import open
    import backports.configparser as configparser
except ImportError:
    import configparser


logger = logging.getLogger(__name__)
logging.basicConfig()


# Main class for util script
class KeyringUtils(object):
    """Our main app class"""

    def __init__(self):
        super(KeyringUtils, self).__init__()

        config_file = resilient.get_config_file()
        print(u"Configuration file: {}".format(config_file))

        # Read configuration options.
        if config_file:
            config_path = resilient.ensure_unicode(config_file)
            config_path = os.path.expanduser(config_path)
            if os.path.exists(config_path):
                try:
                    self.config = configparser.ConfigParser(interpolation=None)
                    with open(config_path, 'r', encoding='utf-8') as f:
                        first_byte = f.read(1)
                        if first_byte != u'\ufeff':
                            # Not a BOM, no need to skip first byte
                            f.seek(0)
                        self.config.read_file(f)
                except Exception as exc:
                    logger.warn(u"Couldn't read config file '%s': %s", config_path, exc)
                    self.config = None
            else:
                logger.warn(u"Couldn't read config file '%s'", config_file)
        else:
            logger.warn(u"Couldn't read config file")

    def run(self):
        """
        Read the configuration file
        For any 'keyring'-based parameters: prompt the user to set or edit the value.
        """
        opts = {"__any_secrets__": False}
        for section in self.config.sections():
            items = dict((item.lower(), self.config.get(section, item)) for item in self.config.options(section))
            items["__any_secrets__"] = False
            opts.update({section: items})

        print(u"Secrets are stored with '{}'".format(type(keyring.get_keyring()).__module__))
        try:
            self.list_parameters(opts)
        except (KeyboardInterrupt, EOFError):
            print()

        if opts.get("__any_secrets__"):
            print(u"Done.")
        else:
            print(u"Nothing to do.")

    @staticmethod
    def list_parameters(options):
        """
        Process all sections in the config 'options' dictionary
        """
        names = ()
        return _list_parameters(names, options)


def get_input(prompt):
    if sys.version_info[0] >= 3:
        return input(prompt)
    value = raw_input(prompt)
    return value.decode(sys.stdin.encoding)


def _list_parameters(names, options):
    """
    Parse parameters, with a tuple of names for keyring context.

    Given a dict that has configuration keys mapped to values,
       - If a value begins with '^', redirect to fetch the value from
         the secret key stored in the keyring.
         The keyring service name is always just an underscore
         (so keys must be unique in the whole options dict)

    """
    for key in options.keys():
        val = options[key]
        if isinstance(val, dict):
            val = _list_parameters(names + (key,), val)
            options["__any_secrets__"] = options["__any_secrets__"] or val["__any_secrets__"]
        if isinstance(val, string_types) and len(val) > 1 and val[0] == "^":
            # This value is from the keyring
            options["__any_secrets__"] = True
            tag = val
            val = val[1:]
            service = ".".join(names) or "_"
            if service == "resilient":
                # Special case, becuase of the way we parse commandlines, treat this as root
                service = "_"
            logger.debug("keyring get('%s', '%s')", service, val)
            value = keyring.get_password(service, val)

            if value is None:
                print("[{0}] {1}: <not set>".format(".".join(names) or "_", key))
            else:
                print("[{0}] {1}: {2}".format(".".join(names) or "_", key, tag))

            newvalue = None
            do_set = True
            while do_set:
                newvalue = get_input(u"  Enter new value (or <ENTER> to leave unchanged): ")
                if len(newvalue) == 0:
                    do_set = False
                    break
                confirm = get_input(u"  Confirm new value: ")
                if confirm == newvalue:
                    break
                print(u"Values do not match, try again.")

            if do_set:
                keyring.set_password(service, val, newvalue)
                print(u"Value set.")

        options[key] = val
    return options


def main():
    KeyringUtils().run()


if __name__ == "__main__":
    main()



# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
from __future__ import print_function
import pytest
import resilient


class TestPatch:
    def test_patch(self):
        existing = {"a": 1, "properties": {"b": 2}, "vers": 99}

        patch = resilient.Patch(existing)

        patch.add_value("a", 5)
        patch.add_value("properties.b", 6)
        patch.add_value("c", 7)

        dto = patch.to_dict()

        assert dto["version"] == 99

        changes = dto["changes"]

        assert len(changes) == 3

        change = changes[0]
        assert change["field"] == "a"
        assert change["old_value"]["object"] == 1
        assert change["new_value"]["object"] == 5

        change = changes[1]
        assert change["field"] == "properties.b"
        assert change["old_value"]["object"] == 2
        assert change["new_value"]["object"] == 6

        change = changes[2]
        assert change["field"] == "c"
        assert not change["old_value"]["object"]
        assert change["new_value"] ["object"]== 7

    def test_partial_property_name(self):
        existing = {"properties": {"a": 5}}
        patch = resilient.Patch(existing)

        with pytest.raises(ValueError) as exception_info:
            patch.add_value("properties", 99)

        assert "Invalid field_name parameter" in str(exception_info.value)

    def test_no_old_value(self):
        patch = resilient.Patch({})

        # this one is allowed
        patch.add_value("a", new_value=5, old_value=3)

        dto = patch.to_dict()

        changes = dto["changes"]

        assert len(changes) == 1
        assert changes[0]["field"] == "a"
        assert changes[0]["old_value"]["object"]== 3
        assert changes[0]["new_value"]["object"] == 5

        with pytest.raises(ValueError) as exception_info:
            patch.add_value("b", new_value=10)

        assert "Constructor previous_object or method old_value argument is required" in str(exception_info.value)

    def test_add_twice(self):
        patch = resilient.Patch({"a": 5})

        patch.add_value("a", 7)
        patch.add_value("a", 8)

        mydict = patch.to_dict()

        changes = mydict["changes"]

        assert len(changes) == 1

        assert changes[0]["field"] == "a"
        assert changes[0]["old_value"]["object"] == 5
        assert changes[0]["new_value"]["object"] == 8

    def test_null_old_value(self):
        patch = resilient.Patch({"blah": "old value"})

        patch.add_value("blah", "new value", old_value=None)

        mydict = patch.to_dict()

        assert mydict

        changes = mydict["changes"]

        assert changes

        assert len(changes) == 1
        assert changes[0]["field"] == "blah"
        assert not changes[0]["old_value"]["object"]
        assert changes[0]["new_value"]["object"]== "new value"

    def test_has_changes(self):
        patch = resilient.Patch(dict(testfield=1))

        assert not patch.has_changes()

        patch.add_value("testfield", 5)

        assert patch.has_changes()

    def test_delete(self):
        patch = resilient.Patch(dict(a=1, b=2))

        assert not patch.has_changes()

        patch.add_value("a", 11)

        assert patch.has_changes()

        assert patch.get_old_value("a") == 1
        assert patch.get_new_value("a") == 11

        patch.delete_value("a")

        assert not patch.has_changes()


class TestPatchStatus:
    @pytest.mark.parametrize("success", (True, False))
    def test_success(self, co3_args, success):
        test_data = {
            "success": success
        }

        status = resilient.PatchStatus(test_data)

        assert status.is_success() == success

    @staticmethod
    def _make_test_data():
        return {
            "success": False,
            "field_failures": [
                {
                    "field": "mytest1",
                    "your_original_value": "original1",
                    "actual_current_value": "current1"
                }, {
                    "field": "mytest2",
                    "your_original_value": "original2",
                    "actual_current_value": "current2"
                }
            ],
            "message": "Some message"
        }

    def test_has_failures(self):
        status = resilient.PatchStatus(TestPatchStatus._make_test_data())

        assert not status.is_success()
        assert status.has_field_failures()
        assert status.get_conflict_fields() == ["mytest1", "mytest2"]

        assert status.is_conflict_field("mytest1")
        assert status.is_conflict_field("mytest2")

        assert not status.is_conflict_field("blah")

    def test_values(self):
        status = resilient.PatchStatus(TestPatchStatus._make_test_data())

        assert status.get_your_original_value("mytest1") == "original1"
        assert status.get_actual_current_value("mytest1") == "current1"

        assert status.get_your_original_value("mytest2") == "original2"
        assert status.get_actual_current_value("mytest2") == "current2"

    def test_field_name_found(self):
        status = resilient.PatchStatus(TestPatchStatus._make_test_data())

        with pytest.raises(ValueError) as exception_info:
            status.get_your_original_value("blah")

        assert "No conflict found for field blah" in str(exception_info.value)

        with pytest.raises(ValueError) as exception_info:
            status.get_your_original_value("blah")

        assert "No conflict found for field blah" in str(exception_info.value)

    def test_message(self):
        status = resilient.PatchStatus(TestPatchStatus._make_test_data())

        assert status.get_message() == "Some message"

    def test_exchange_conflicting_value(self):
        # Given a base object with a value of "test1".
        base = dict(mytest1 = "test1")

        # And a patch that is attempting to modify that base object to have a value of "test2".
        patch = resilient.Patch(base)

        patch.add_value("mytest1", "test2")

        # Confirm that it does indeed have an "old value" of "test1" (this is taken from the base object).
        assert patch.get_old_value("mytest1") == "test1"

        # When we create a patch status that simulates a conflict error from the server (where the
        # value of base changed from "test1" to "blah").
        patch_status = resilient.PatchStatus({
            "success": False,
            "field_failures": [
                {
                    "field": "mytest1",
                    "your_original_value": "test2",
                    "actual_current_value": "blah"
                }
            ],
            "message": "Some message"
        })

        # When I exchange the conflicting value...
        patch.exchange_conflicting_value(patch_status, "mytest1", "test2")

        # The patch's "old value" will be the current server's value.
        assert patch.get_old_value("mytest1") == "blah"
        assert patch.get_new_value("mytest1") == "test2"







# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.

"""Base client for Resilient REST API"""
from __future__ import print_function

import json
import ssl
import mimetypes
import os
import sys
import logging
import unicodedata
import requests

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager
from requests_toolbelt.multipart.encoder import MultipartEncoder

try:
    # Python 3
    import urllib.parse as urlparse
except:
    # Python 2
    import urlparse

LOG = logging.getLogger(__name__)


class TLSHttpAdapter(HTTPAdapter):
    """
    Adapter that ensures that we use the best available SSL/TLS version.
    Some environments default to SSLv3, so we need to specifically ask for
    the highest protocol version that both the client and server support.
    Despite the name, SSLv23 can select "TLS" protocols as well as "SSL".
    """
    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(num_pools=connections,
                                       maxsize=maxsize,
                                       block=block,
                                       ssl_version=ssl.PROTOCOL_SSLv23)


class BasicHTTPException(Exception):
    """Exception for HTTP errors."""
    def __init__(self, response):
        """
        Args:
          response - the Response object from the get/put/etc.
        """
        super(BasicHTTPException, self).__init__(u"{0}:  {1}".format(response.reason, response.text))

        self.response = response

    def get_response(self):
        return self.response

    @staticmethod
    def raise_if_error(response):
        if response.status_code != 200:
            raise BasicHTTPException(response)


class NoChange(Exception):
    """Exception that can be raised within a get/put handler or a patch callback
       to indicate 'no change' (which then just bypasses the update operation).
    """
    pass


def ensure_unicode(input_value):
    """ if input_value is type str, convert to unicode with utf-8 encoding """
    if sys.version_info.major >= 3:
        return input_value

    if not isinstance(input_value, basestring):
        return input_value
    elif isinstance(input_value, str):
        input_unicode = input_value.decode('utf-8')
    else:
        input_unicode = input_value

    input_unicode = unicodedata.normalize('NFKC', input_unicode)
    return input_unicode


def get_proxy_dict(opts):
    """ Creates a dictionary with proxy config to be sent to the SimpleClient """
    scheme = urlparse.urlparse(opts.proxy_host).scheme
    if not scheme:
        scheme = 'https'
        proxy_host = opts.proxy_host
    else:
        proxy_host = opts.proxy_host[len(scheme + "://"):]

    if opts.proxy_user and opts.proxy_password:
        proxy = {'https': '{0}://{1}:{2}@{3}:{4}/'.format(scheme, opts.proxy_user, opts.proxy_password,
                                                          proxy_host, opts.proxy_port)}
    else:
        proxy = {'https': '{0}://{1}:{2}'.format(scheme, proxy_host, opts.proxy_port)}

    return proxy


class BaseClient(object):
    """Helper for using Resilient REST API."""

    def __init__(self, org_name=None, base_url=None, proxies=None, verify=None):
        """
        Args:
          org_name - the name of the organization to use.
          base_url - the base URL to use.
          proxies - HTTP proxies to use, if any.
          verify - The name of a PEM file to use as the list of trusted CAs.
        """
        self.headers = {'content-type': 'application/json'}
        self.cookies = None
        self.org_id = None
        self.user_id = None
        self.base_url = u'https://app.resilientsystems.com/'
        self.org_name = ensure_unicode(org_name)
        if proxies:
            self.proxies = {ensure_unicode(key): ensure_unicode(proxies[key]) for key in proxies}
        else:
            self.proxies = None
        if base_url:
            self.base_url = ensure_unicode(base_url)
        self.verify = verify
        self.verify = ensure_unicode(verify)
        if verify is None:
            self.verify = True
        self.authdata = None
        self.session = requests.Session()
        self.session.mount(u'https://', TLSHttpAdapter())

    def connect(self, email, password, timeout=None):
        """Performs connection, which includes authentication.

        Args:
          email - the email address to use for authentication.
          password - the password
          timeout - number of seconds to wait for response
        Returns:
          The Resilient session object (dict)
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        self.authdata = {
            u'email': ensure_unicode(email),
            u'password': ensure_unicode(password)
        }
        return self._connect(timeout=timeout)

    def _connect(self, timeout=None):
        """Establish a session"""
        response = self.session.post(u"{0}/rest/session".format(self.base_url),
                                     data=json.dumps(self.authdata),
                                     proxies=self.proxies,
                                     headers=self.make_headers(),
                                     verify=self.verify,
                                     timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        session = json.loads(response.text)
        orgs = session['orgs']
        selected_org = None
        if orgs is None or len(orgs) == 0:
            raise Exception("User is a member of no orgs")
        elif self.org_name:
            org_names = []
            for org in orgs:
                org_name = org['name']
                org_names.append(org_name)
                if ensure_unicode(org_name) == self.org_name:
                    selected_org = org
        else:
            org_names = [org['name'] for org in orgs]
            msg = u"Please specify the organization name to which you want to connect.  " + \
                  u"The user is a member of the following organizations: '{0}'"
            raise Exception(msg.format(u"', '".join(org_names)))

        if selected_org is None:
            msg = u"The user is not a member of the specified organization '{0}'."
            raise Exception(msg.format(self.org_name))

        if not selected_org.get("enabled", False):
            msg = "This organization is not accessible to you.\n\n" \
                  "This can occur because of one of the following:\n\n" \
                  "The organization does not allow access from your current IP address.\n" \
                  "The organization requires authentication with a different provider than you are currently using.\n" \
                  "Your IP address is {0}"
            raise Exception(msg.format(session["session_ip"]))

        self.all_orgs = [org for org in orgs if org.get("enabled")]
        self.org_id = selected_org['id']

        # set the X-sess-id token, which is used to prevent CSRF attacks.
        self.headers['X-sess-id'] = session['csrf_token']
        self.cookies = {
            'JSESSIONID': response.cookies['JSESSIONID']
        }
        self.user_id = session["user_id"]
        return session

    def make_headers(self, co3_context_token=None, additional_headers=None):
        """Makes a headers dict, including the X-Co3ContextToken (if co3_context_token is specified)."""
        headers = self.headers.copy()
        if co3_context_token is not None:
            headers['X-Co3ContextToken'] = co3_context_token
        if isinstance(additional_headers, dict):
            headers.update(additional_headers)
        return headers

    def _execute_request(self, operation, url, **kwargs):
        """Execute a HTTP request.
           If unauthorized (likely due to a session timeout), retry.
        """
        result = operation(url, **kwargs)
        if result.status_code == 401:  # unauthorized, re-auth and try again
            self._connect()
            result = operation(url, **kwargs)
        return result

    def get(self, uri, co3_context_token=None, timeout=None):
        """Gets the specified URI.  Note that this URI is relative to <base_url>/rest/orgs/<org_id>.  So
        for example, if you specify a uri of /incidents, the actual URL would be something like this:

            https://app.resilientsystems.com/rest/orgs/201/incidents

        Args:
          uri
          co3_context_token
          timeout: number of seconds to wait for response
        Returns:
          A dictionary or array with the value returned by the server.
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        response = self._execute_request(self.session.get,
                                         url,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        return json.loads(response.text)

    def get_content(self, uri, co3_context_token=None, timeout=None):
        """Gets the specified URI.  Note that this URI is relative to <base_url>/rest/orgs/<org_id>.  So
        for example, if you specify a uri of /incidents, the actual URL would be something like this:

            https://app.resilientsystems.com/rest/orgs/201/incidents

        Args:
          uri
          co3_context_token
          timeout: number of seconds to wait for response
        Returns:
          The raw value returned by the server for this resource.
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        response = self._execute_request(self.session.get,
                                         url,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        return response.content

    def post(self, uri, payload, co3_context_token=None, timeout=None):
        """
        Posts to the specified URI.
        Note that this URI is relative to <base_url>/rest/orgs/<org_id>.  So for example, if you
        specify a uri of /incidents, the actual URL would be something like this:

            https://app.resilientsystems.com/rest/orgs/201/incidents
        Args:
           uri
           payload
           co3_context_token
          timeout: number of seconds to wait for response
        Returns:
          A dictionary or array with the value returned by the server.
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        payload_json = json.dumps(payload)
        response = self._execute_request(self.session.post,
                                         url,
                                         data=payload_json,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        return json.loads(response.text)

    def post_attachment(self, uri, filepath,
                        filename=None, mimetype=None, data=None, co3_context_token=None, timeout=None):
        """
        Upload a file to the specified URI
        e.g. "/incidents/<id>/attachments" (for incident attachments)
        or,  "/tasks/<id>/attachments" (for task attachments)

        :param uri: The REST URI for posting
        :param filepath: the path of the file to post
        :param filename: optional name of the file when posted
        :param mimetype: optional override for the guessed MIME type
        :param data: optional dict with additional MIME parts (not required for file attachments; used in artifacts)
        :param co3_context_token: Action Module context token, if responding to an Action Module event
        :param timeout: optional timeout (seconds)
        """
        filepath = ensure_unicode(filepath)
        if filename:
            filename = ensure_unicode(filename)
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        mime_type = mimetype or mimetypes.guess_type(filename or filepath)[0] or "application/octet-stream"
        with open(filepath, 'rb') as filehandle:
            attachment_name = filename or os.path.basename(filepath)
            multipart_data = {'file': (attachment_name, filehandle, mime_type)}
            multipart_data.update(data or {})
            encoder = MultipartEncoder(fields=multipart_data)
            headers = self.make_headers(co3_context_token,
                                        additional_headers={'content-type': encoder.content_type})
            response = self._execute_request(self.session.post,
                                             url,
                                             data=encoder,
                                             proxies=self.proxies,
                                             cookies=self.cookies,
                                             headers=headers,
                                             verify=self.verify,
                                             timeout=timeout)
            BasicHTTPException.raise_if_error(response)
            return json.loads(response.text)

    def post_artifact_file(self, uri, artifact_type, artifact_filepath,
                           description=None, value=None, mimetype=None, co3_context_token=None, timeout=None):
        """
        Post a file artifact to the specified URI
        e.g. "/incidents/<id>/artifacts/files"

        :param uri: The REST URI for posting
        :param artifact_type: the artifact type name ("IP Address", etc) or type ID
        :param artifact_filepath: the path of the file to post
        :param description: optional description for the artifact
        :param value: optional value for the artifact
        :param mimetype: optional override for the guessed MIME type
        :param co3_context_token: Action Module context token, if responding to an Action Module event
        :param timeout: optional timeout (seconds)

        """
        artifact = {
            "type": artifact_type,
            "value": value or "",
            "description": description or ""
        }
        mimedata = {
            "artifact": json.dumps(artifact)
        }
        return self.post_attachment(uri,
                                    artifact_filepath,
                                    mimetype=mimetype,
                                    data=mimedata,
                                    co3_context_token=co3_context_token,
                                    timeout=timeout)

    def _get_put(self, uri, apply_func, co3_context_token=None, timeout=None):
        """Internal helper to do a get/apply/put loop
        (for situations where the put might return a 409/conflict status code)
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        response = self._execute_request(self.session.get,
                                         url,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        payload = json.loads(response.text)
        try:
            apply_func(payload)
        except NoChange:
            return payload
        payload_json = json.dumps(payload)
        response = self._execute_request(self.session.put,
                                         url,
                                         data=payload_json,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        if response.status_code == 200:
            return json.loads(response.text)
        elif response.status_code == 409:
            return None
        BasicHTTPException.raise_if_error(response)
        return None

    def get_put(self, uri, apply_func, co3_context_token=None, timeout=None):
        """Performs a get, calls apply_func on the returned value, then calls self.put.
        If the put call returns a 409 error, then retry.

        Args:
          uri - the URI to use.  Note that this is expected to be relative to the org.
          apply_func - a function to call on the object returned by get.  This is expected
          to alter the object with the desired changes.
          co3_context_token - the Co3ContextToken from a CAF message (if the caller is
          a CAF message processor.
          timeout - number of seconds to wait for response
        Returns;
          The object returned by the put operation (converted from JSON to a Python dict).
        Raises:
          Exception if the get or put returns an unexpected status code.
        """
        while True:
            obj = self._get_put(uri, apply_func, co3_context_token=co3_context_token, timeout=timeout)
            if obj:
                return obj
        return None

    def put(self, uri, payload, co3_context_token=None, timeout=None):
        """
        Puts to the specified URI.
        Note that this URI is relative to <base_url>/rest/orgs/<org_id>.  So for example, if you
        specify a uri of /incidents, the actual URL would be something like this:

            https://app.resilientsystems.com/rest/orgs/201/incidents
        Args:
           uri
           payload
           co3_context_token
          timeout: number of seconds to wait for response
        Returns:
          A dictionary or array with the value returned by the server.
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        payload_json = json.dumps(payload)
        response = self._execute_request(self.session.put,
                                         url,
                                         data=payload_json,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        BasicHTTPException.raise_if_error(response)
        return json.loads(response.text)

    def delete(self, uri, co3_context_token=None, timeout=None):
        """Deletes the specified URI.

        Args:
          uri
          co3_context_token
          timeout: number of seconds to wait for response
        Returns:
          A dictionary or array with the value returned by the server.
        Raises:
          BasicHTTPException - if an HTTP exception occurs.
        """
        url = u"{0}/rest/orgs/{1}{2}".format(self.base_url, self.org_id, ensure_unicode(uri))
        response = self._execute_request(self.session.delete,
                                         url,
                                         proxies=self.proxies,
                                         cookies=self.cookies,
                                         headers=self.make_headers(co3_context_token),
                                         verify=self.verify,
                                         timeout=timeout)
        if response.status_code == 204:
            # 204 - No content is OK for a delete
            return None
        BasicHTTPException.raise_if_error(response)
        return json.loads(response.text)



# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
from __future__ import print_function
import pytest
import doctest
import time
import types
import os
import resilient


class TestCo3:
    """Basic API tests"""

    def test_doctest(self):
        """Run doctest on all modules, even if you didn't --doctest-modules"""
        for item in resilient.__dict__.values():
            if type(item) is types.ModuleType:
                doctest.testmod(item, verbose=True, raise_on_error=True)

    def test_connect_no_verify(self, co3_args):
        """ Successful connection with no Cert Verification """
        url = "https://{0}:{1}".format(co3_args.host, co3_args.port or 443)
        client = resilient.SimpleClient(org_name=co3_args.org,
                                        base_url=url,
                                        verify=False)
        assert client
        userinfo = client.connect(co3_args.email, co3_args.password)
        assert userinfo

    def test_connect_proxy(self, co3_args):
        """ Successful connection with no Cert Verification """
        url = "https://{0}:{1}".format(co3_args.host, co3_args.port or 443)
        client = resilient.SimpleClient(org_name=co3_args.org,
                                        base_url=url,
                                        verify=False,
                                        proxies=co3_args.proxy)
        assert client
        user_info = client.connect(co3_args.email, co3_args.password)
        assert user_info


class TestCo3Patch:
    """Tests for patch, create_patch, and get_patch methods"""
    def _connect(self, co3_args):
        uri = "https://{0}:{1}".format(co3_args.host, co3_args.port or 443)
        client = resilient.SimpleClient(org_name=co3_args.org,
                                        base_url=uri,
                                        verify=False)
        assert client
        user_info = client.connect(co3_args.email, co3_args.password)
        assert user_info
        return client

    def _create_incident(self, client, incident_dict):
        """create a test incident to use for patching"""
        incident = {"name": __name__,
                    "discovered_date": int(time.time() * 1000)
        }
        incident.update(incident_dict)
        inc = client.post("/incidents", incident)
        return inc

    @pytest.mark.parametrize("overwrite_conflict", (True, False))
    def test_patch_conflict(self, co3_args, overwrite_conflict):
        """ do incident patch that results in conflict """
        client = self._connect(co3_args)
        inc = self._create_incident(client, {"name": "test"})
        uri = "/incidents/%d" % inc['id']
        # Create a conflict
        inc["name"] = "the wrong value"
        inc["vers"] -= 1 # Force it to check old_value
        patch = resilient.Patch(inc)
        patch.add_value("name", "test updated")

        if overwrite_conflict:
            # If overwrite_conflict is specified then patch will return.
            response = client.patch(uri, patch, overwrite_conflict=overwrite_conflict)

            assert resilient.PatchStatus(response.json()).is_success() is overwrite_conflict
        else:
            # Not overwriting conflict, so an exception will be thrown.
            with pytest.raises(resilient.PatchConflictException) as exception_info:
                client.patch(uri, patch, overwrite_conflict=overwrite_conflict)

            # Gather the patch_status value from the exception for additional verification.
            patch_status = exception_info.value.patch_status

            fail_msg = "could not be applied due to a conflicting edit by another user.  The following field(s) were in conflict:  name."
            assert fail_msg in patch_status.get_message()

            assert patch_status.get_conflict_fields() == ["name"]
            assert patch_status.get_your_original_value("name") == "the wrong value"
            assert patch_status.get_actual_current_value("name") == "test"

        inc = client.get("/incidents/%d" % inc['id'])

        if overwrite_conflict:
            assert inc['name'] == "test updated"
        else:
            assert inc['name'] == "test"

    def test_patch_no_conflict(self, co3_args):
        """ do incident_patch with no conflict """
        client = self._connect(co3_args)
        inc = self._create_incident(client, {"name": "test"})
        uri = "/incidents/%d" % inc['id']
        patch = resilient.Patch(inc)
        patch.add_value("name", "test updated")
        response = client.patch(uri, patch, overwrite_conflict=False)
        assert resilient.PatchStatus(response.json()).is_success()
        inc = client.get("/incidents/%d" % inc['id'])
        assert inc['name'] == "test updated"

    def test_patch_null_old_value(self, co3_args):
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test", "description": None})

        patch = resilient.Patch(inc)

        patch.add_value("description", "new value")

        uri = "/incidents/%d" % inc['id']

        response = client.patch(uri, patch)

        inc = client.get("/incidents/%d" % inc['id'])

        assert inc["description"] == "new value"

    def test_patch_invalid_callback(self, co3_args):
        """
        If a callback returns True but didn't modify the passed in patch in any way, that'd be a problem.
        So make sure we throw an exception in that case.
        """
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test"})

        uri = "/incidents/%d" % inc['id']

        # Create a conflict
        inc["name"] = "the wrong value"
        inc["vers"] -= 1 # Force it to check old_value

        patch = resilient.Patch(inc)

        patch.add_value("name", "test updated")

        def mycb(response, patch_status, patch):
            # Return True but don't modify the patch.
            return True

        with pytest.raises(ValueError) as exception_info:
            client.patch_with_callback(uri, patch, mycb)

        assert "invoked callback did not change the patch object, but returned True" in str(exception_info.value)

    def test_no_change(self, co3_args):
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test"})

        uri = "/incidents/%d" % inc['id']

        # Create a conflict
        inc["name"] = "the wrong value"
        inc["vers"] -= 1 # Force it to check old_value

        patch = resilient.Patch(inc)

        patch.add_value("name", "test updated")

        def mycb(response, patch_status, patch):
            raise resilient.NoChange

        response = client.patch_with_callback(uri, patch, mycb)

        assert response
        assert response.status_code == 200

        patch_status = resilient.PatchStatus(response.json())

        assert not patch_status.is_success()
        assert patch_status.get_conflict_fields() == ["name"]

    def test_conflict_with_handler(self, co3_args):
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test"})

        uri = "/incidents/%d" % inc['id']

        # Create a conflict
        inc["name"] = "the wrong value"
        inc["vers"] -= 1 # Force it to check old_value

        patch = resilient.Patch(inc)

        patch.add_value("name", "test updated")

        def mycb(response, patch_status, patch):
            patch.exchange_conflicting_value(patch_status, "name", "test updated take 2")

        response = client.patch_with_callback(uri, patch, mycb)

        assert response
        assert response.status_code == 200

        assert "test updated take 2" == client.get(uri)["name"]

    def test_delete(self, co3_args):
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test"})

        uri = "/incidents/%d" % inc['id']

        response = client.delete(uri)
        assert response
        assert response['success'] == True

    def test_post_attachment(self, co3_args):
        client = self._connect(co3_args)

        inc = self._create_incident(client, {"name": "test for attachment"})

        file_name = "test-for-attachment.txt"
        file_content = "this is test data"

        # Create the file
        temp_file = open(file_name, "w")
        temp_file.write(file_content)
        temp_file.close()
        # Post file to Resilient
        response = client.post_attachment("/incidents/{0}/attachments".format(inc["id"]),
                                          file_name,
                                          file_name,
                                          mimetype="text/plain")

        assert response
        assert response['name'] == file_name

        os.remove(file_name)

    def test_get_config_file(self, co3_args):
        config_file = resilient.get_config_file("~/.resilient/app.config")

        assert config_file

    def test_get_client(self, co3_args):
        client = resilient.get_client(co3_args)

        assert client

    def test_throw_simple_http_exception(self, co3_args):
        with pytest.raises(resilient.SimpleHTTPException) as exception_info:
            client = resilient.SimpleClient("Not A real Org")
            user_info = client.connect("not_a_user@mail.com", "test")

            assert exception_info

    def test_get_const(self, co3_args):
        client = self._connect(co3_args)

        response = client.get_const()

        assert response

    def test_get_content(self, co3_args):
        client = self._connect(co3_args)

        response = client.get_content("/incidents")

        assert response

    def test_get_proxy_dict(self, co3_args):
        proxy_opts = type('', (), {})()
        proxy_opts.proxy_host = "http://resilientproxy.com"
        proxy_opts.proxy_port = "4443"
        proxy_opts.proxy_user = None
        proxy_opts.proxy_password = None
        proxy_dict = resilient.get_proxy_dict(proxy_opts)

        assert proxy_dict == {'https': 'http://resilientproxy.com:4443'}

        proxy_opts.proxy_user = "user"
        proxy_opts.proxy_password = "password"
        proxy_dict = resilient.get_proxy_dict(proxy_opts)

        assert proxy_dict == {'https': 'http://user:password@resilientproxy.com:4443/'}

        proxy_opts.proxy_host = "resilientproxy.com"

        proxy_dict = resilient.get_proxy_dict(proxy_opts)

        assert proxy_dict == {'https': 'https://user:password@resilientproxy.com:4443/'}





# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
import pytest
import unittest
import mock
from resilient.bin.res_keyring import KeyringUtils

class TestKeyringUtils(unittest.TestCase):
    mocked_resilient_class = mock.Mock()
    mocked_resilient_get_config = mock.Mock()
    @mock.patch("resilient.get_config_file", mocked_resilient_get_config)
    def test_run(self):
        mocked_get_config = mock.Mock()
        mocked_get_config = self.mocked_resilient_class.get_config_file.return_value

        keyring_util = KeyringUtils()
        keyring_util.run()

        assert keyring_util






# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
""" pytest configuration for co3 tests """
import argparse
import pytest


def pytest_addoption(parser):
    parser.addoption("--config-file",
                     help="Resilient Config File",
                     default="",
                     required=False)
    parser.addoption('--co3args',
                     help="Optional Resilient Args",
                     default="",
                     required=False)


@pytest.fixture(scope="session")
def co3_args(request):
    print("co3_args fixture")
    import shlex
    import resilient

    config_file = request.config.getoption("--config-file")
    co3args = request.config.getoption("--co3args")
    co3args = shlex.split(co3args)

    args = resilient.ArgumentParser(config_file=config_file).parse_args(args=co3args)
    return args



# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
import pytest
import unittest
import mock
from resilient.bin.gadget import ExampleArgumentParser, \
    generic_get, generic_post, generic_update, generic_patch, generic_search


class TestExampleArgumentParser(unittest.TestCase):
    def test_run(self):

        exampleAP = ExampleArgumentParser(config_file=None)

        assert exampleAP

    def test_generic_get(self):
        mocked_rest_client = mock.Mock()
        mocked_get_call = mock.Mock()
        mocked_rest_client.get = mocked_get_call.return_value
        mocked_rest_client.get.side_effect = client_return
        mocked_get_call.__len__ = lambda s: 100

        generic_get(mocked_rest_client, "test")

    def test_generic_post(self):
        mocked_rest_client = mock.Mock()
        mocked_post_call = mock.Mock()
        mocked_rest_client.post = mocked_post_call.return_value
        mocked_rest_client.post.side_effect = client_return
        mocked_post_call.__len__ = lambda s: 100

        generic_post(mocked_rest_client, "test", "template_test.json")

    def test_generic_update(self):
        mocked_rest_client = mock.Mock()
        mocked_get_put_call = mock.Mock()
        mocked_rest_client.get_put = mocked_get_put_call.return_value
        mocked_rest_client.get_put.side_effect = client_return
        mocked_get_put_call.__len__ = lambda s: 100

        generic_update(mocked_rest_client, "test", "template_test.json")

    def test_generic_patch(self):
        mocked_rest_client = mock.Mock()
        mocked_patch_call = mock.Mock()
        mocked_rest_client.patch = mocked_patch_call.return_value
        mocked_rest_client.patch.side_effect = client_return
        mocked_patch_call.__len__ = lambda s: 100

        generic_patch(mocked_rest_client, "test", "template_test.json")

    def test_generic_search(self):
        mocked_rest_client = mock.Mock()
        mocked_search_call = mock.Mock()
        mocked_rest_client.search = mocked_search_call.return_value
        mocked_rest_client.search.side_effect = client_return
        mocked_search_call.__len__ = lambda s: 100

        generic_search(mocked_rest_client, "template_test.json")

def client_return(*args, **kwargs):
    return {"id": "id"}


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.

""" Setup for resilient module """

from __future__ import print_function
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
from pkg_resources import get_distribution, DistributionNotFound

try:
    # pip version 10 onward
    from pip._internal.utils.misc import get_installed_distributions
except ImportError:
    from pip import get_installed_distributions


def check_deps():
    # Fail if the 'co3' module is installed, this supersedes it
    packages = get_installed_distributions(local_only=True)
    # For each EggInfoDistribution, find its metadata
    for pkg in packages:
        try:
            distro = get_distribution(pkg.project_name)
            if distro.project_name == 'co3':
                print("This package replaces the 'co3' distribution.  Please 'pip uninstall co3' first.")
                sys.exit(1)
        except DistributionNotFound:
            pass


if __name__ == "__main__":
    check_deps()


class PyTest(TestCommand):
    user_options = [('configfile=', 'c', "Resilient Config File for co3argparse"),
                    ('co3args=', 'a', "Resilient Optional Args for co3argparse"),
                    ('pytestargs=', 'p', "Pytest Optional Args")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.configfile = ""
        self.co3args = ""
        self.pytestargs = ""
        self.test_suite = True
        self.test_args = []

    def finalize_options(self):
        import shlex
        TestCommand.finalize_options(self)
        if self.configfile:
            self.test_args += ["--config-file=%s" % self.configfile]
        if self.co3args:
            self.test_args += ["--co3args=%s" % self.co3args]
        if self.pytestargs:
            self.test_args += shlex.split(self.pytestargs)

    def run_tests(self):
        # import here, because outside the eggs aren't loaded
        print("Running Tests with args: %s" % self.test_args)
        import pytest
        errno = pytest.main(args=self.test_args)
        sys.exit(errno)


setup(
    name='resilient',
    use_scm_version={"root": "../", "relative_to": __file__},
    setup_requires=['setuptools_scm'],
    url='https://developer.ibm.com/resilient',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    author='IBM Resilient',
    install_requires=[
        'argparse',
        'requests>=2.6.0',
        'requests-toolbelt>=0.6.0',
        'requests-mock>=1.2.0',
        'six',
        'cachetools<3.0.0'
    ],
    extras_require={
        ':python_version < "3.2"': [
            'configparser'
        ],
        ':"Debian" in platform_version': [
            'keyring<=9.1'
            # There is no 'gcc' on Resilient appliance; later versions cause trouble
        ],
        ':python_version >= "2.7"': [
            'keyring'
        ],
        ':python_version == "2.6"': [
            'keyring==5.4'
        ]
    },
    tests_require=["pytest", ],
    cmdclass={"test": PyTest},
    author_email='support@resilientsystems.com',
    description='Resilient API',
    long_description='Resilient API Modules for Python',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': ['finfo = resilient.bin.finfo:main',
                            'gadget = resilient.bin.gadget:main',
                            'res-keyring = resilient.bin.res_keyring:main']
    }
)



# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
import pytest
import unittest
import mock
from resilient.bin.finfo import FinfoArgumentParser, print_details, find_field, list_fields_values, list_fields_csv,list_fields, list_types



class TestFinfoArgumentParser(unittest.TestCase):
    def test_print_details(self):
        print_details({"id": "1", "name": "test", "text": "test", "value": "1234", "input_type": "test",
                       "prefix": "test", "tooltip": "test", "placeholder": "test"})

    def test_find_field(self):
        mocked_rest_client = mock.Mock()
        mocked_get_call = mock.Mock()
        mocked_rest_client.get = mocked_get_call.return_value
        mocked_rest_client.get.side_effect = client_return
        mocked_get_call.__len__ = lambda s: 100

        field = find_field(mocked_rest_client, "test")
        assert field


    def test_list_fields_values(self):
        mocked_rest_client = mock.Mock()
        mocked_get_call = mock.Mock()
        mocked_rest_client.get = mocked_get_call.return_value
        mocked_rest_client.get.side_effect = client_return
        mocked_get_call.__len__ = lambda s: 100

        list_fields_values(mocked_rest_client)

    def test_list_fields_csv(self):
        mocked_rest_client = mock.Mock()
        mocked_get_call = mock.Mock()
        mocked_rest_client.get = mocked_get_call.return_value
        mocked_rest_client.get.side_effect = client_return
        mocked_get_call.__len__ = lambda s: 100

        list_fields_csv(mocked_rest_client)

    def test_list_fields(self):
        mocked_rest_client = mock.Mock()
        mocked_get_call = mock.Mock()
        mocked_rest_client.get = mocked_get_call.return_value
        mocked_rest_client.get.side_effect = client_return
        mocked_get_call.__len__ = lambda s: 100

        list_fields(mocked_rest_client)


def client_return(*args, **kwargs):
    inStr = str(args[0])
    ret = []
    if inStr == "/types":
        ret = [{"test": {"id": "1", "name": "test", "text": "test", "value": "1234", "input_type": "test",
                "prefix": "test", "tooltip": "test", "placeholder": "test"}}]
    elif inStr == "/types/incident/fields":
        ret = [{"id": "1", "name": "test", "text": "test", "value": "1234", "input_type": "test",
                "prefix": "test", "tooltip": "test", "placeholder": "test"}]
    return ret